@@ -9,3 +9,7 @@ package tsconst
 // interfaces on Windows. This is set by the WinTun driver.
 const WintunInterfaceDesc = "Tailscale Tunnel"
 const WintunInterfaceDesc0_14 = "Wintun Userspace Tunnel"
+
+// TailnetLockNotTrustedMsg is the error message used by network lock
+// and sniffed (via substring) out of an error sent over the network.
+const TailnetLockNotTrustedMsg = "this node is not trusted by network lock"
